# -*- coding: latin1 -*-
#
# Copyright (C) 2003 Ayose Cazorla Len
#
# Authors
#       Ayose Cazorla <ayose.cazorla@hispalinux.es>
#
# This file is part of Pyslide.
#
# Pyslide is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Pyslide is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pyslide; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
                                                                                                

import misc
class InvalidCSS(misc.PyslideError):
    'This CSS is not valid'

class InvalidXML(misc.PyslideError):
    'This XML file is not valid'

import re
re_css_class = re.compile(r'^\s*?\.(\S+)\s*\{(\s*.+?\s*)\}(.*)$', re.S)

def parse_css(css_text):
    css = {}
    while True:
        m = re_css_class.match(css_text)
        if not m:
            return css

        # new class
        class_name = m.group(1).lower()
        content = m.group(2)
        css_text = m.group(3) 

        # parse attributes
        d = {}
        for line in content.split(';'):
            line = line.strip()
            if not line: continue
            try:
                key, val = line.split(':')
            except ValueError:
                raise InvalidCSS

            d[key.strip().lower()] = val.strip()

        css[class_name] = d

import xml.sax
import xml.sax.saxutils

from Content import Group, Page, Content

class ContentParser(xml.sax.saxutils.DefaultHandler):
    def __init__(self):
        xml.sax.saxutils.DefaultHandler.__init__(self)

        self.__content = Content()
        self.depth = 0

        self.last_page = None
        self.last_item = None
        self.last_group = None

        self.current_tag = None

        self.last_group_attrs = {}
        self.styles = {}
        self.style_chars = ''

    def startElement(self, name, xml_attrs):
        self.depth += 1
        self.current_tag = name

        # Copy the attributes into a dictionary
        attrs = {}
        for key in xml_attrs.keys():
            val = xml_attrs[key]
            if key == 'class':
                if not self.styles.has_key(val):
                    raise InvalidXML, '%s: No such class' % val

                for a, b in self.styles[val].items():
                    attrs[a] = b

            else:
                attrs[key] = val

        if name == 'presentation':
            if self.depth != 1:
                raise InvalidXML, 'The root node has to be <presentation>'

            # Save the presentation attributes.
            self.__content.set_attrs(attrs)

        elif name == 'page':
            if self.depth != 2:
                raise InvalidXML, '<page> has to be a child of the root node'

            self.last_page = self.__content.add_page(attrs)

        elif name == 'group':
            if self.depth != 3:
                raise InvalidXML, '<group> has to be a child of a page'

            self.last_group = self.last_page.add_group([])
            self.last_group_attrs = attrs

        elif name in ('text', 'image', 'shape'):
            if self.depth != 4:
                raise InvalidXML, '<%s> has to be a child of a group' % name

            # copy groups attributes
            for key, val in self.last_group_attrs.items():
                if not attrs.has_key(key):
                    attrs[key] = val

            self.last_item = \
                self.last_group.add_item({'type': name, 'attrs': attrs, 'content': ''})

        elif name == 'style':
            # limited support for CSS. I put a pass here because, if not,
            # an InvalidXML exception will be raised.
            pass

        else:
            raise InvalidXML, 'Unknown tag: "%s"' % name

    def endElement(self, name):
        if self.current_tag == 'style':
            for css_class, attributes  in parse_css(self.style_chars).items():
                if self.styles.has_key(css_class):
                    dest = self.styles[css_class]
                else:
                    dest = self.styles[css_class] = {}

                for key, val in attributes.items():
                    dest[key] = val

            self.style_chars = ''

        self.current_tag = None
        self.depth -= 1

    def characters(self, chrs):
        if self.current_tag == 'style':
            self.style_chars += chrs

        elif self.depth == 4:
            self.last_item['content'] = chrs

    def get_content(self):
        return self.__content

def ReadFile(filename):

    parser = xml.sax.make_parser()
    dh = ContentParser()
    parser.setContentHandler(dh)
    try:
        parser.parse(filename)
    except xml.sax.SAXException, e:
        raise InvalidXML, e

    return dh.get_content()


def WriteFile(file, content):

    # XXX perhaps do this with pyxml?

    def write_open(node, attrs, marg=0, keep=False):
        if marg: file.write(' ' * marg)
        file.write('<' + node)
        for key, value in attrs.items():
            file.write(' %s="%s"' % (key, value))
        file.write(keep and '>' or '>\n')

    def write_close(node, marg=0):
        if marg: file.write(' ' * marg)
        file.write('</%s>\n' % node)

    # Main attributes
    write_open('presentation', content.get_attrs())

    # Pages
    for page in content.pages():
        write_open('page', page.get_attrs(), 4)

        # its items
        for group in page.groups():
            write_open('group', {}, 8)
            for item in group.items():
                write_open(item['type'], item['attrs'], 12, True)
                file.write(item['content'])
                write_close(item['type'])
            write_close('group', 8)

        write_close('page', 4)

    write_close('presentation')


def Compile(content, output, fullscreen=False):
    import time

    w = output.write
    w('''\
#!/usr/bin/python\n# -*- coding: latin1 -*-
# This is a compiled pyslide presentation
# Created on %s

''' % time.strftime('%c'))

    import os.path
    path, N = os.path.split(__file__)
    del N
    path = path[:path.rfind('/')]

    w('''\
PYSLIDE_PATH = '%s'
FULLSCREEN = %s

import sys
if PYSLIDE_PATH not in sys.path:
    sys.path.append(PYSLIDE_PATH)

''' % (path, repr(fullscreen)))

    w('''\
from Pyslide import Presentation as P
from Pyslide import Content as C

c = C.Content()
c.set_attrs(%s)
''' % repr(content.get_attrs()))

    for page in content.pages():
        w('\n# %New page%\n')
        w('p = c.add_page(%s)\n' % repr(page.get_attrs()))
        for group in page.groups():
            w('g = p.add_group()         # %Group%\n')
            for item in group.items():
                w('g.add_item(%s)\n' % repr(item))

    w('\n\n#Run presentation\nP.BuildPresentation(c, FULLSCREEN).run()\n')

if __name__ == '__main__':
    import sys
    try:
        content = ReadFile(sys.argv[1])
        print ' * Presentation'
        for attr in content.get_attrs().items():
            print ' -  %s="%s"' % attr


        for n, page in enumerate(content.pages()):
            print ' ** Page #%d' % n
            for attr in page.get_attrs().items():
                print '  -   %s="%s"' % attr


            for n, group in enumerate(page.groups()):
                print ' *** Group #%d' % n
                for n, item in enumerate(group.items()):
                    print '   - Item:'
                    for attr in item.items():
                        print '        %s="%s"' % attr


    except InvalidXML, e:
        print 'Invalid XML file:', e


