# -*- coding: latin1 -*-
#
# Copyright (C) 2003 Ayose Cazorla Len
#
# Authors
#       Ayose Cazorla <ayose.cazorla@hispalinux.es>
#
# This file is part of Pyslide.
#
# Pyslide is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Pyslide is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pyslide; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import pygame
from misc import parse_color, load_image, PyslideError
from pygame.locals import *

_has_init = False

def NewPresentation(caption='Presentation', width=640, height=480, flags=HWACCEL):
    global _has_init
    if not _has_init:
        pygame.init()
        _has_init = True

    win = pygame.display.set_mode((width, height), flags)
    pygame.display.set_caption(caption)
    return Presentation(win)


class Presentation:
    def __init__ (self, win):
        self.win   = win
        self.pages = []
        self.num_page   = 0
        self.need_display_flip = False
        self.current_page = pygame.sprite.RenderUpdates()

    def add_sprites(self, sprites):
        for s in sprites:
            s.start()
            self.current_page.add(s)

    def remove_sprites(self, sprites, force_stop=False):
        if force_stop:
            for s in sprites: s.force_stop()
        else:
            for s in sprites: s.stop()

    def add_page (self, page):
        self.pages.append (page)
        return page

    def next_page (self):
        if self.num_page < len(self.pages)-1:
            self.set_page(self.num_page + 1)

    def prev_page (self):
        if self.num_page > 0:
            self.set_page(self.num_page - 1, show_effects = False)

    def set_page(self, n, show_effects=True):
        self.num_page = n

        # empty the page
        self.remove_sprites(self.current_page.sprites(), not show_effects)

        # add the new sprites
        page = self.pages[self.num_page]
        for group in page.current_groups():
            self.add_sprites(group)

        # show the page
        self.win.blit(page.bg, (0,0))
        #pygame.display.flip()
        self.need_display_flip = True

        import time
        self.page_start_time = time.time()

    def run (self):
        self.set_page(0)
        #clock = pygame.time.Clock()
        import time
        while True:
            #clock.tick(20)
            time.sleep(0.05)
            page = self.pages[self.num_page]

            self.current_page.clear(self.win, page.bg)
            self.current_page.update()
 
            for event in pygame.event.get():
                if event.type == QUIT:
                    return
                elif event.type is KEYDOWN:
                    if event.key == K_f:
                        pygame.display.toggle_fullscreen()
                    elif event.key == K_LEFT:
                        self.prev_stage()
                    elif event.key in (K_RIGHT, K_SPACE):
                        self.next_stage()
                    elif event.key == K_PAGEUP:
                        self.pages[self.num_page].set_stage(0)
                        self.prev_page()
                    elif event.key == K_PAGEDOWN:
                        self.pages[self.num_page].set_stage(-1)
                        self.next_page()
                    elif event.key == K_q:
                        return

            if page.ttl is not None:
                if time.time() - self.page_start_time >= page.ttl:
                    self.pages[self.num_page].set_stage(-1)
                    self.next_page()

            dirty = self.current_page.draw(self.win)
            pygame.display.update(dirty)

            if self.need_display_flip:
                pygame.display.flip()
                self.need_display_flip = False


            # if the page is empty, go to the next stage
            if len(self.current_page) == 0:
                self.next_stage()

    def next_stage(self):
        page = self.pages[self.num_page]
        sprites = page.next_stage()
        if sprites is None:
            self.next_page()
        else:
            self.add_sprites(sprites)

    def prev_stage(self):
        page = self.pages[self.num_page]
        sprites = page.prev_stage()
        if sprites is None:
            self.prev_page()
        else:
            self.remove_sprites(sprites, force_stop = True)


def BuildPresentation(content, fullscreen=False):
    '''BuildPresentation (content) -> present

    Parses the content data and creates a new presentation.
    content is the result of File.ReadFile.
    '''

    present_attrs = content.get_attrs()

    flags = HWACCEL
    if fullscreen: flags |= FULLSCREEN
    # create the presentation

    try:
        w, h = [int(x) for x in present_attrs['size'].split('x')]
    except ValueError:
        raise misc.PyslideError, 'Invalid size ("%s")' % present_attrs['size']
    except KeyError:
        w, h = 640, 480

    presentation = NewPresentation (
        present_attrs.get('caption', 'Presentation'), w, h, flags)

    # have a default background?
    if present_attrs.has_key('bg'):
        default_background = load_image(present_attrs['bg']).convert()
        default_background = pygame.transform.scale(default_background,
                             pygame.display.get_surface().get_size())
    else:
        # set a default background
        if present_attrs.has_key('bgcolor'):
            color = parse_color(present_attrs['bgcolor'])
        else:
            color = (0,0,0)
        default_background = pygame.Surface(pygame.display.get_surface().get_size())
        default_background.fill(color)

    # Create the pages
    from Pages import Page
    import Items
    for xml_page in content.pages():
        page = Page()
        presentation.add_page(page)
        attrs  = xml_page.get_attrs()
        Items.LAST_Y = 0

        if attrs.has_key('ttl'):
            try:
                page.set_ttl(int(attrs['ttl']) / 10.)
            except ValueError:
                raise PyslideError, 'Invalid TTL value: ' + str(attrs['ttl'])

        # Background
        try:
            if attrs.has_key('bg'):
                page.SetBG(attrs['bg'], scale = attrs.get('bgscale', 'yes') == 'yes')
            elif attrs.has_key('bgcolor'):
                color = parse_color(attrs['bgcolor'])
                if color:
                    page.SetBGColor(color)
                else:
                    raise RuntimeError

            elif attrs.has_key('bggrad'):
                    top, bottom = attrs['bggrad'].split('-')
                    top = parse_color(top)
                    bottom = parse_color(bottom)

                    if None in (top, bottom):
                        raise RuntimeError

                    page.SetBGGrad(top, bottom);
            else:
                raise RuntimeError

        except (RuntimeError, TypeError):
            # default background: white
            page.bg = default_background

        # Items
        for group in xml_page.groups():
            page.new_group()

            for item in group.items():
                what = item['type']
                if what == 'text':
                    # Text items
                    state = Items.ItemText(item['content'], item['attrs'])
                    page.add_item(state)

                elif what == 'image':
                    # Image (pixmaps) items
                    page.add_item(Items.ItemImage(item['attrs']))

                elif what == 'shape':
                    # Shape items
                    page.add_item(Items.ItemShape(item['attrs']))

                else:
                    raise PyslideError, 'Unknow element: ' + str(item['type'])

    return presentation

